

(********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2002             *
 * by the XIPHOPHORUS Company http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

 function: simple example encoder
 last mod: $Id: encoder_example.c,v 1.50 2002/07/16 09:26:07 xiphmont Exp $

 ********************************************************************)

//* takes a mono 16bit 44.1kHz WAV file and encodes it into a Vorbis bitstream */

{ =============================================

  Delphi conversion: Aleksandr "Lake" Shamray
		     http://lakeofsoft.com/

		     last modified: 13 Feb 2004

 =============================================}

{$APPTYPE CONSOLE}

{$Q-,B-,O+,R-,I-}

program
  encoder_example;

uses
  unaVorbisAPI;

const
  READ = 1024;	// number of samples to read at once 

var
  readbuffer: array[0..READ * 2 - 1] of smallInt; //* take space out of the data segment, not the stack */
  fileIn, fileOut: file;

procedure run();
var
  os: tOgg_stream_state; //* take physical pages, weld into a logical stream of packets */
  og: tOgg_page; //* one Ogg bitstream page.  Vorbis packets are inside */
  op: tOgg_packet; //* one raw packet of data for encode */

  vi: tVorbis_info; //* struct that stores all the static vorbis bitstream settings */
  vc: tVorbis_comment; //* struct that stores all the user comments */

  vd: tVorbis_dsp_state; //* central working state for the packet->PCM decoder */
  vb: tVorbis_block; //* local working space for packet->PCM decode */

  header: tOgg_packet;
  header_comm: tOgg_packet;
  header_code: tOgg_packet;

  eos, ret: int;
  i: int;

  readCount, writeCount: int;

  buffer: pSingleSamples;

  inCount, outCount: int64;
begin
  eos := 0;
  inCount := 0;
  outCount := 0;

  (* we cheat on the WAV header; we just bypass the first 44 bytes and never
     verify that it matches 16bit/mono/44.1kHz.  This is just an
     example, after all. *)

  blockRead(fileIn, readbuffer, 44, readCount);

  //********** Encode setup ************/

  vorbis_info_init(vi);

  //* choose an encoding mode.  A few possibilities commented out, one actually used: */

  (*********************************************************************
   Encoding using a VBR quality mode.  The usable range is -.1
   (lowest quality, smallest file) to 1. (highest quality, largest file).
   Example quality mode .4: 44kHz stereo coupled, roughly 128kbps VBR

   ret = vorbis_encode_init_vbr(&vi,2,44100,.4);

   ---------------------------------------------------------------------

   Encoding using an average bitrate mode (ABR).
   example: 44kHz stereo coupled, average 128kbps VBR

   ret = vorbis_encode_init(&vi,2,44100,-1,128000,-1);

   ---------------------------------------------------------------------

   Encode using a qulity mode, but select that quality mode by asking for
   an approximate bitrate.  This is not ABR, it is true VBR, but selected
   using the bitrate interface, and then turning bitrate management off:

   ret = ( vorbis_encode_setup_managed(&vi,2,44100,-1,128000,-1) ||
	   vorbis_encode_ctl(&vi,OV_ECTL_RATEMANAGE_AVG,NULL) ||
	   vorbis_encode_setup_init(&vi));

   *********************************************************************)

  ret := vorbis_encode_init_vbr(vi, 1, 44100, 0.5);

  (* do not continue if setup failed; this can happen if we ask for a
     mode that libVorbis does not support (eg, too low a bitrate, etc,
     will return 'OV_EIMPL') *)

  if (0 <> ret) then
    exit;

  //* add a comment */
  vorbis_comment_init(vc);
  vorbis_comment_add_tag(vc, 'ENCODER', 'encoder_example_mono.dpr');

  //* set up the analysis state and auxiliary encoding storage */
  vorbis_analysis_init(vd, vi);
  vorbis_block_init(vd, vb);

  //* set up our packet->stream encoder */
  //* pick a random serial number; that way we can more likely build chained streams just by concatenation */
  randomize();
  ogg_stream_init(os, random(maxInt));

  (* Vorbis streams begin with three headers; the initial header (with
     most of the codec setup parameters) which is mandated by the Ogg
     bitstream spec.  The second header holds any comment fields.  The
     third header holds the bitstream codebook.  We merely need to
     make the headers, then pass them to libvorbis one at a time;
     libvorbis handles the additional Ogg bitstream constraints *)
  begin
    vorbis_analysis_headerout(vd, vc, header, header_comm, header_code);
    ogg_stream_packetin(os, header); //* automatically placed in its own page */
    ogg_stream_packetin(os, header_comm);
    ogg_stream_packetin(os, header_code);

    //* This ensures the actual audio data will start on a new page, as per spec */
    while (0 <> ogg_stream_flush(os, og)) do begin
      blockWrite(fileOut, og.header^, og.header_len, writeCount);
      inc(outCount, writeCount);
      blockWrite(fileOut, og.body^, og.body_len, writeCount);
      inc(outCount, writeCount);
    end;
  end;

  // - do encode -
  while(0 = eos) do begin
    //
    blockRead(fileIn, readbuffer, READ shl 2, readCount);
    inc(inCount, readCount);

    if (0 = readCount) then begin
      (* end of file.  this can be done implicitly in the mainline,
	 but it's easier to see here in non-clever fashion.
	 Tell the library we're at end of stream so that it can handle
	 the last frame and mark end of stream in the output properly *)
      vorbis_analysis_wrote(vd, 0);
    end
    else begin
      //* data to encode */

      //* expose the buffer to submit data */
      buffer := vorbis_analysis_buffer(vd, READ);

      //* uninterleave samples */
      i := 0;
      while (i < readCount shr 2) do begin
	buffer[0][i] := (readbuffer[i shl 1] + readbuffer[i shl 1 + 1]) / 65536;
	inc(i);
      end;

      //* tell the library how much we actually submitted */
      vorbis_analysis_wrote(vd, i);
    end;

    (* vorbis does some data preanalysis, then divvies up blocks for
       more involved (potentially parallel) processing.  Get a single
       block for encoding now *)
    while (1 = vorbis_analysis_blockout(vd, vb)) do begin

      //* analysis, assume we want to use bitrate management */
      vorbis_analysis(vb, nil);
      vorbis_bitrate_addblock(vb);

      while (0 <> vorbis_bitrate_flushpacket(vd, op)) do begin

	//* weld the packet into the bitstream */
	ogg_stream_packetin(os, op);

	//* write out pages (if any) */
	while (0 = eos) do begin
	  //
	  if (0 = ogg_stream_pageout(os, og)) then
	    break;
	  //
	  blockWrite(fileOut, og.header^, og.header_len, writeCount);
	  inc(outCount, writeCount);
	  blockWrite(fileOut, og.body^, og.body_len, writeCount);
          inc(outCount, writeCount);

	  //* this could be set above, but for illustrative purposes, I do it here (to show that vorbis does know where the stream ends) */
	  if (0 <> ogg_page_eos(og)) then
	    eos := 1;
	end
      end
    end;

    //
    write('input: ', inCount, ';   output: ', outCount, '      '#13);
  end;

  //* clean up and exit.  vorbis_info_clear() must be called last */
  ogg_stream_clear(os);
  vorbis_block_clear(vb);
  vorbis_dsp_clear(vd);
  vorbis_comment_clear(vc);
  vorbis_info_clear(vi);

  //* ogg_page and ogg_packet structs always point to storage in libvorbis. They're never freed or manipulated directly */

  writeLn('');
  writeLn('Done.');
end;

// -- main --
begin
  if (paramCount < 2) then begin
    writeLn('must specify input (wav) and output (ogg) file names.');
    exit;
  end
  else begin
    //
    if (vorbis_load_library(cunav_dll_ogg) and vorbis_load_library(cunav_dll_vorbis) and vorbis_load_library(cunav_dll_vorbisenc)) then begin
      //
      assignFile(fileIn, paramStr(1));
      reset(fileIn, 1);		// record size = 1
      //
      assignFile(fileOut, paramStr(2));
      rewrite(fileOut, 1);	// record size = 1
      //
      try
	run();
      finally
	//
	closeFile(fileIn);
	closeFile(fileOut);
	//
	vorbis_unload_library(cunav_dll_vorbisenc);
	vorbis_unload_library(cunav_dll_vorbis);
	vorbis_unload_library(cunav_dll_ogg);
      end;
    end
    else
      writeLn('No vorbis encoder library was found.');
  end;
end.

